#ifndef _DIALOG_H_
#define _DIALOG_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 


#include <qcheckbox.h>
#include <qcombobox.h>
#include <qdialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qradiobutton.h>
#include <qspinbox.h> 
#include <qstring.h> 

#include "hexbuffer.h"
#include "hexvalidator.h"

const int repYes   = QDialog::Accepted; 
const int repNo    = 11;
const int repAll   = 12;
const int repClose = QDialog::Rejected; 



class CGotoDialog : public QDialog 
{
  Q_OBJECT
  
  public:
    CGotoDialog( QWidget *parent, const char *name = 0 );
    ~CGotoDialog( void );
    void defaultFocus( void );

  signals:
    void execute( USearchData &sd );

  private slots:
    void startGoto( void );

  private:
    QComboBox *mComboBox;
    QCheckBox *mCheckBackward;
    QCheckBox *mCheckFromCursor;  
};


class CFindDialog : public QDialog 
{
  Q_OBJECT

  public:
    CFindDialog( QWidget *parent, const char *name = 0 );
    ~CFindDialog( void );
 
    void defaultFocus( void );
    const char *data( void );

  signals: 
    void execute( USearchData &sd );

  public slots:
    void findAgain( void );

  private slots:
    void startFind( void );
    void selectorChanged( int index );
    void inputChanged( const char *text );

  private:
    uint size( void );

  private:
    QComboBox *mSelector;
    QLineEdit *mInput;
    QCheckBox *mCheckBackward;
    QCheckBox *mCheckFromCursor;
    QCheckBox *mCheckInSelection;

    QString    mFindString[4];
    QByteArray mFindData;
    CHexValidator *mFindValidator;
};


class CReplaceDialog : public QDialog 
{
  Q_OBJECT
  
  public:
    CReplaceDialog( QWidget *parent, const char *name = 0 );
    ~CReplaceDialog( void );

    void defaultFocus( void );

  signals:
    void execute( USearchData &sd );

  private slots:
    void startReplace( void );
    void findSelectorChanged( int index );
    void findInputChanged( const char *text );
    void replaceSelectorChanged( int index );
    void replaceInputChanged( const char *text );

  private:
    const char *findData( void );
    uint findSize( void );
    const char *replaceData( void );
    uint replaceSize( void );

  private:
    QComboBox *mFindSelector;
    QComboBox *mReplaceSelector;
    QLineEdit *mFindInput;
    QLineEdit *mReplaceInput;

    QCheckBox *mCheckBackward;
    QCheckBox *mCheckFromCursor;
    QCheckBox *mCheckInSelection;
    QCheckBox *mCheckPrompt;

    QString    mFindString[4];
    QString    mReplaceString[4];
    QByteArray mFindData;
    QByteArray mReplaceData;
    CHexValidator *mFindValidator;
    CHexValidator *mReplaceValidator;
};






class CReplacePromptDialog : public QDialog 
{
  Q_OBJECT
  
  public:
    CReplacePromptDialog( QWidget *parent, const char *name = 0 );
  
  signals:
    void execute( int returnCode );

  private slots:
    void no( void );
    void yes( void );
    void all( void );
    void close( void );

  private:
    void done( int returnCode );

};





class CInsertDialog : public QDialog 
{
  Q_OBJECT
  
  public:
    CInsertDialog( QWidget *parent, const char *name = 0 );
    ~CInsertDialog( void );
    void defaultFocus( void );

  signals:
    void execute( SInsertData &id );

  private slots:
    void startInsert( void );
    void cursorCheck( void );
    void patternSelectorChanged( int index );
    void patternInputChanged( const char *text );

  private:
    const char *pattern( void );
    uint patternSize( void );

  private:
    QSpinBox *mSizeBox;
    QLabel *mOffsetLabel;
    QComboBox *mPatternSelector;
    QLineEdit *mPatternInput;
    QLineEdit *mOffsetInput;
    QCheckBox *mCheckPattern;
    QCheckBox *mCheckOnCursor;

    QString    mPatternString[4];
    QByteArray mPatternData;
    CHexValidator *mPatternValidator;
};





class CExportDialog : public QDialog 
{
  Q_OBJECT
  
  public:
    CExportDialog( QWidget *parent, const char *name = 0 );
    ~CExportDialog( void );
    void defaultFocus( void );

  signals:
    void execute( SExportData &ed );

  private slots:
    void startExport( void );
    void radioClicked( int id );
    void checkStdout( void );

  private:
    QLabel *mDestinationLabel;
    QLineEdit *mDestinationInput;
    QRadioButton *mRadioEverything;
    QRadioButton *mRadioSelectedArea;
    QRadioButton *mRadioSpecify;
    QLabel *mFromLabel;
    QLabel *mToLabel;
    QLineEdit *mFromInput;
    QLineEdit *mToInput;
    QCheckBox *mCheckPS;
    QCheckBox *mCheckStdout;
};













void centerDialog( QWidget *widget, QWidget *centerParent );
bool stringToOffset( const char *text, uint &offset );


#endif


