#include <iostream.h>

#include <qdir.h> 
#include <qmsgbox.h> 

#include <kapp.h>
#include <kfiledialog.h>
#include <kurl.h>

#include "hexeditorwidget.h"




CHexEditorWidget::CHexEditorWidget( QWidget *parent, const char *name )
  : QWidget( parent, name )
{
  CHexBuffer *hexBuffer = new CHexBuffer;
  if( hexBuffer == 0 ) { return; }
  
  mDocumentList.setAutoDelete( true );
  mDocumentList.append( hexBuffer );
  mUntitledCount = 1;

  mHexView = new CHexViewWidget( this, hexBuffer );
  if( mHexView == 0 || mHexView->widgetValid() == false ) { return; }

  connect( mHexView, SIGNAL(openNewFile()), SLOT( newFile()) );
  connect( kapp, SIGNAL( kdisplayFontChanged() ), SLOT( fontChanged() ) );
  connect( kapp, SIGNAL( kdisplayPaletteChanged() ),SLOT( paletteChanged()) );
  connect( mHexView, SIGNAL( layoutChanged( const SDisplayLayout & ) ),
	   SLOT( layoutChanged( const SDisplayLayout & ) ) );
  mHexView->setFocus();
  setBackgroundColor( kapp->windowColor );

  mKfm = 0;
  mGotoDialog = 0;
  mFindDialog = 0;
  mReplaceDialog = 0;
  mInsertDialog  = 0;
  mExportDialog = 0;
}

CHexEditorWidget::~CHexEditorWidget( void )
{
  delete mHexView;
  delete mGotoDialog;
  delete mFindDialog;
  delete mReplaceDialog;
  delete mInsertDialog;
  delete mExportDialog;
  delete mOptionDialog;
}


void CHexEditorWidget::writeConfiguration( KConfig &config )
{
  SDisplayLayout &d = mDisplayState.layout;

  config.setGroup( "Display Options" );
  config.writeEntry( "PrimaryMode",          d.primaryModeString() );
  config.writeEntry( "SecondaryMode",        d.secondaryModeString() );
  config.writeEntry( "OffsetMode",           d.offsetModeString() );
  config.writeEntry( "OffsetVisible",        d.offsetVisible );
  config.writeEntry( "PrimaryUpperCase",     d.primaryUpperCase );
  config.writeEntry( "OffsetUpperCase",      d.offsetUpperCase );
  config.writeEntry( "LineSize",             d.lineSize );
  config.writeEntry( "ColumnSize",           d.columnSize );
  config.writeEntry( "LockLine",             d.lockLine );
  config.writeEntry( "LockColumn",           d.lockColumn );
  config.writeEntry( "ColumnSpacing",        d.columnSpacing );
  config.writeEntry( "SeparatorMarginWidth", d.separatorMarginWidth );
  config.writeEntry( "EdgeMarginWidth",      d.edgeMarginWidth );
  config.writeEntry( "LeftSeparatorWidth",   d.leftSeparatorWidth );
  config.writeEntry( "RightSeparatorWidth",  d.rightSeparatorWidth );
}

void CHexEditorWidget::readConfiguration( KConfig &config )
{
  SDisplayLayout &d = mDisplayState.layout;

  config.setGroup( "Display Options" );
  d.setPrimaryMode( config.readEntry("PrimaryMode") );
  d.setSecondaryMode( config.readEntry("SecondaryMode") );
  d.setOffsetMode( config.readEntry("OffsetMode") );
  d.offsetVisible        = config.readBoolEntry( "OffsetVisible", true );
  d.primaryUpperCase     = config.readBoolEntry( "PrimaryUpperCase", false );
  d.offsetUpperCase      = config.readBoolEntry( "OffsetUpperCase", false );
  d.lineSize             = config.readNumEntry( "LineSize", 16 );
  d.columnSize           = config.readNumEntry( "ColumnSize", 1 );
  d.lockLine             = config.readBoolEntry( "LockLine", false );
  d.lockColumn           = config.readBoolEntry( "LockColumn", false );
  d.columnSpacing        = config.readNumEntry( "ColumnSpacing", 1 );
  d.separatorMarginWidth = config.readNumEntry( "SeparatorMarginWidth", 7 );
  d.edgeMarginWidth      = config.readNumEntry( "EdgeMarginWidth", 5 );
  d.leftSeparatorWidth   = config.readNumEntry( "LeftSeparatorWidth", 1 );
  d.rightSeparatorWidth  = config.readNumEntry( "RightSeparatorWidth", 1 );
}



void CHexEditorWidget::resizeEvent( QResizeEvent *e )
{
  mHexView->resize( e->size().width(),  e->size().height() );
}


void CHexEditorWidget::fontChanged( void )
{
  if( mDisplayState.font.useSystemFont == true )
  {
    mHexView->setFont( kapp->fixedFont, true );
  }
}

void CHexEditorWidget::paletteChanged( void )
{
  setColor( mDisplayState.color );
}

void CHexEditorWidget::layoutChanged( const SDisplayLayout &layout )
{
  mDisplayState.layout = layout;
}


void CHexEditorWidget::setLineSize(const SDisplayLine &line )
{
  mDisplayState.line = line;
}

void CHexEditorWidget::setLayout( const SDisplayLayout &layout )
{
  mDisplayState.layout.lockLine = layout.lockLine;
  mDisplayState.layout.lockColumn = layout.lockColumn;
  mDisplayState.layout.leftSeparatorWidth = layout.leftSeparatorWidth;
  mDisplayState.layout.rightSeparatorWidth = layout.rightSeparatorWidth;
  mDisplayState.layout.separatorMarginWidth = layout.separatorMarginWidth;
  mDisplayState.layout.edgeMarginWidth = layout.edgeMarginWidth;

  SDisplayLayout &l = mDisplayState.layout;
  l.lineSize = mDisplayState.line.lineSize[ l.primaryMode ];
  l.columnSize = mDisplayState.line.columnSize[ l.primaryMode ];

  mHexView->setLayout( mDisplayState.layout );
}
  
void CHexEditorWidget::setCursor( const SDisplayCursor &cursor )
{
  mDisplayState.cursor = cursor;
  mHexView->setCursor( mDisplayState.cursor, true );
}

void CHexEditorWidget::setColor( const SDisplayColor &color )
{
  mDisplayState.color = color;

  mDisplayState.color.selectBg = kapp->selectColor;
  mDisplayState.color.selectFg = kapp->selectTextColor;

  if( mDisplayState.color.useSystemColor == true )
  {
    SDisplayColor c = mDisplayState.color;
    c.inactiveBg = kapp->windowColor;
    c.primaryFg[0] = kapp->windowTextColor;
    c.primaryFg[1] = kapp->windowTextColor;
    c.offsetFg = kapp->windowTextColor;
    c.secondaryFg = kapp->windowTextColor;
    c.leftSeparatorFg = kapp->windowTextColor;
    c.rightSeparatorFg = kapp->windowTextColor;
    c.cursorBg = kapp->windowTextColor;
    c.cursorFg = kapp->windowColor;
    mHexView->setColor( c, true ); 
  }
  else
  {
    mHexView->setColor( mDisplayState.color, true ); 
  }
}


void CHexEditorWidget::setFont( const SDisplayFont &font )
{
  mDisplayState.font = font;
  if( mDisplayState.font.useSystemFont == true )
  {
    mHexView->setFont( kapp->fixedFont, true );
  }
  else
  {
    mHexView->setFont( mDisplayState.font.localFont, true );
  }
}

void CHexEditorWidget::setMisc( const SDisplayMisc &misc )
{
  mDisplayState.misc = misc;
  mHexView->setMisc( mDisplayState.misc );
}



void CHexEditorWidget::newFile( void )
{
  const char *newName = i18n( "Untitled" );
  QString url( strlen( newName ) + 20 );
  url.sprintf( "%s %u", newName, mUntitledCount );
  
  //
  // If the url is already present in the document list (should not happen), 
  // then this document is "raised". 
  //
  bool success = selectDocument( url );
  if( success == true )
  {
    return;
  }
  
  //
  // Prepare a new buffer we can load the document into
  //
  success = createBuffer();
  if( success == false )
  {
    return;
  }

  mHexView->loadFile( 0, url, false );
  mUntitledCount += 1;
}


void CHexEditorWidget::open( void )
{
  QString url = KFileDialog::getOpenFileURL( 0, "*", topLevelWidget() );
  if( url.isEmpty() == true )
  {
    return;
  }

  //
  // If the url is already present in the document list, then this
  // document is "raised". 
  //
  bool success = selectDocument( url );
  if( success == true )
  {
    return;
  }

  //
  // Prepare a new buffer we can load the document into
  //
  success = createBuffer();
  if( success == false )
  {
    return;
  }

  readURL( url, false );
}



void CHexEditorWidget::open( QString &url )
{
  //
  // If the url is already present in the document list, then this
  // document is "raised". 
  //
  bool success = selectDocument( url );
  if( success == true )
  {
    return;
  }
  
  //
  // Prepare a new buffer we can load the document into
  //
  success = createBuffer();
  if( success == false )
  {
    return;
  }

  readURL( url, false );
}




bool CHexEditorWidget::selectDocument( QString &url )
{
  CHexBuffer *hexBuffer = documentItem( url );
  if( hexBuffer == 0 )
  {
    return( false );
  }

  mHexView->setBuffer( hexBuffer );
  return( true );
}


void CHexEditorWidget::insertFile( void )
{
  QString url = KFileDialog::getOpenFileURL( 0, "*", topLevelWidget(), 0 );
  if( url.isEmpty() == true )
  {
    return;
  }

  readURL( url, true );
}



void CHexEditorWidget::stop( void )
{
  QMessageBox::warning(
    topLevelWidget(),
    i18n("Stop"),
    i18n("Sorry, not yet implemented"),
    i18n("OK"),
    "",
    "",
    0,0);
}


bool CHexEditorWidget::close( void )
{
  if( querySave() == false )
  {
    return( false );
  }

  removeBuffer();
  return( true );
}




bool CHexEditorWidget::closeAll( void )
{
  while( mDocumentList.count() > 0 )
  {
    CHexBuffer *buf;
    for( buf = mDocumentList.first(); buf != 0; buf = mDocumentList.next() )
    {
      //
      // We select the buffer we are about to close becuase it will
      // the become visible, and because 'close()' works on the current
      // buffer.
      //
      mHexView->setBuffer( buf );
      if( close() == false )
      {
	return( false );
      }
      break;
    }

    //
    // The document list will always contain at least one element because
    // the hexview widget needs a hexbuffer to work. If this hexbuffer is
    // not valid, then all files have been closed.
    //
    if( mDocumentList.count() == 1 && mHexView->valid() == false )
    {
      break;
    }
  }

  return( true );
}












bool CHexEditorWidget::querySave( void )
{
  if( mHexView->modified() == false )
  {
    return( true );
  }

  QString msg;
  msg += i18n( "The current document has been modified." );
  msg += "\n";
  msg += i18n( "Do you want to save it?" );
      
  int reply = QMessageBox::warning( 
    topLevelWidget(),
    i18n( "Warning" ), 
    msg.data(),
    i18n( "&Yes" ),
    i18n( "&No" ),
    i18n( "&Cancel" ),
    0,2 );
  if( reply == 0 ) // Yes
  {
    save();
  }
  else if( reply == 1 )
  {
  }
  else
  {
    return( false );
  }

  return( true );
}







void CHexEditorWidget::save( void )
{
  if( mHexView->valid() == false )
  {
    return;
  }
  if( mHexView->urlValid() == true )
  {
    writeURL( mHexView->url() ); 
  }
  else
  {
    saveAs();
  }
}

void CHexEditorWidget::saveAs( void )
{ 
  if( mHexView->valid() == false )
  {
    return;
  }

  QString url; 
  int query = 0;
  do 
  {
    url = KFileDialog::getSaveFileURL( mHexView->url().data(), "*", this );
    if( url.isEmpty() == true )
    {
      return;
    }

    KURL u(url);
    if( u.isLocalFile() == true ) 
    {
      QFileInfo info;
      QString name( u.path() );
      KURL::decodeURL( name );
      info.setFile( name );

      if( info.exists() == true ) 
      {
	QString msg;
	msg += i18n( "A document with this name already exists." );
	msg += "\n";
	msg += i18n( "Do you want to overwrite it?" );

        query = QMessageBox::warning(
          this,
          i18n( "Warning" ),
	  msg.data(),
          i18n( "Yes" ),
          i18n( "No" ),
          "",
	  0, 1 );
      }
    }
  } 
  while( query == 1 );

  mHexView->setUrl( url );
  writeURL( mHexView->url() ); 

}



void CHexEditorWidget::reload( void )
{
  if( mHexView->valid() == false )
  {
    return;
  }

  if( mHexView->urlValid() == false )
  {
    QString msg;
    msg += i18n( "The current document does not exist on disk." );
  
    QMessageBox::warning(
      topLevelWidget(),
      i18n( "Reload" ),
      msg.data(),
      i18n( "OK" ),
      "",
      "",
      0, 0 );
    return;
  }

  if( mHexView->modified() == true )
  {
    QString msg;
    msg += i18n( "The current document has been modified." );
    msg += "\n";
    msg += i18n( "Do you still want to reload it?" );

    int reply = QMessageBox::warning( 
      topLevelWidget(),
      i18n( "Warning" ), 
      msg.data(),
      i18n( "&Yes" ),
      i18n( "&No" ),
      0, 0 );
    if( reply == 1 ) // No
    {
      return;
    }
  }

  QString url( mHexView->url().data() );
  mHexView->closeFile();
  readURL( url, false );
}



void CHexEditorWidget::print( void )
{
   QMessageBox::warning(
    topLevelWidget(),
    i18n("Print"),
    i18n("Sorry, not yet implemented"),
    i18n("OK"),
    "",
    "",
    0,0);
}




void CHexEditorWidget::export( void )
{
  if( mExportDialog == 0 )
  {
    mExportDialog = new CExportDialog( 0, "Export" );
    if( mExportDialog == 0 ) { return; }
    mExportDialog->setCaption( i18n("Export") );

    connect( mExportDialog, SIGNAL(execute( SExportData & )), 
	     SLOT( exportData( SExportData & )) );
  }
  if( mExportDialog->isVisible() == false )
  {
    centerDialog( mExportDialog, topLevelWidget() );
    mExportDialog->defaultFocus();
    mExportDialog->show();
  }
  else
  {
    mExportDialog->raise();
  }
}


void CHexEditorWidget::exportData( SExportData &ed )
{
  QByteArray array;
  ed.success = mHexView->export( array, ed );
  if( ed.success == false )
  {
    return;
  }

  QFile file;
  if( ed.toStdout == true )
  {
    ed.success = file.open( IO_WriteOnly, stdout );
  }
  else
  {
    file.setName( ed.url.data() );
    ed.success = file.open( IO_WriteOnly );
  }
  if( ed.success == false )
  {
    QString msg = i18n( "Unable to open file for writing" );
    msg += "\n\n";
    msg += file.name();

    QMessageBox::warning(
      mExportDialog,
      i18n("Export"),
      msg.data(),
      i18n("OK"),
      "",
      "",
      0,0 );
    return;
  }

  int writeSize = file.writeBlock( array.data(), array.size() );
  if( writeSize == -1 )
  {
    ed.success = false;
    QString msg = i18n( "An error occured when writing to file" );
    msg += "\n\n";
    msg += file.name();

    QMessageBox::warning(
      mExportDialog,
      i18n("Export"),
      msg.data(),
      i18n("OK"),
      "",
      "",
      0,0 );
    return;
  }

  file.close();
}


void CHexEditorWidget::undo( void )
{
  mHexView->undo();
}

void CHexEditorWidget::redo( void )
{
  mHexView->redo();
}



void CHexEditorWidget::benchmark( void )
{
  mHexView->benchmark();
}

void CHexEditorWidget::copy( void )
{
  mHexView->copy();
}

void CHexEditorWidget::copyPrimaryFormatted( void )
{
  mHexView->copyFormatted( false );
}

void CHexEditorWidget::copyAllFormatted( void )
{
  mHexView->copyFormatted( true );
}

void CHexEditorWidget::paste( void )
{
  if( mHexView->valid() == false )
  {
    pasteNewFile();
  }
  else
  {
    mHexView->paste();
  }
}

void CHexEditorWidget::pasteNewFile( void )
{
  newFile();
  mHexView->paste();
}

void CHexEditorWidget::cut( void )
{
  mHexView->cut();
}

void CHexEditorWidget::selectAll( void )
{
  mHexView->selectAll();
}

void CHexEditorWidget::unselect( void )
{
  mHexView->unselect( 0 );
}


void CHexEditorWidget::addBookmark( void )
{
  mHexView->addBookmark( -1 );
}

void CHexEditorWidget::removeBookmark( void )
{
  if( mHexView->bookmarkCount() == 0 )
  {
    return;
  }

  QString msg;
  msg += i18n( "Deleted bookmarks can not be restored." );
  msg += "\n";
  msg += i18n( "Proceed?" );

  int reply = QMessageBox::information( 
    topLevelWidget(),
    i18n( "Warning" ), 
    msg.data(),
    i18n( "&Yes" ),
    i18n( "&No" ),
    0, 
    0, 1 );
  if( reply == 1 ) // No
  {
    return;
  }

  mHexView->removeBookmark();
}      

void CHexEditorWidget::replaceBookmark( void )
{
  mHexView->replaceBookmark();
}      

void CHexEditorWidget::gotoBookmark( uint position )
{
  mHexView->gotoBookmark( position );
}   




void CHexEditorWidget::gotoOffset( void )
{
  if( mGotoDialog == 0 )
  {
    mGotoDialog = new CGotoDialog( 0, "goto" );
    if( mGotoDialog == 0 ) { return; }
    mGotoDialog->setCaption( i18n("Goto") );
    connect( mGotoDialog, SIGNAL(execute( USearchData & )), 
	     mHexView, SLOT(search( USearchData & )) );
  }
  if( mGotoDialog->isVisible() == false )
  {
    centerDialog( mGotoDialog, topLevelWidget() );
    mGotoDialog->defaultFocus();
    mGotoDialog->show();
  }
  else
  {
    mGotoDialog->raise();
  }
}


void CHexEditorWidget::find( void )
{
  if( mFindDialog == 0 )
  {
    mFindDialog = new CFindDialog( 0, "find" );
    if( mFindDialog == 0 ) { return; }
    mFindDialog->setCaption( i18n("Find") );
    connect( mFindDialog, SIGNAL(execute( USearchData & )), 
	     mHexView, SLOT(search( USearchData & )) );
  }
  if( mFindDialog->isVisible() == false )
  {
    centerDialog( mFindDialog, topLevelWidget() );
    mFindDialog->defaultFocus();
    mFindDialog->show();
  }
  else
  {
    mFindDialog->raise();
  }
}


void CHexEditorWidget::findAgain( void )
{
  if( mFindDialog == 0 || mFindDialog->data() == 0 )
  {
    QString msg;
    msg += i18n("No search pattern is defined.");

    QMessageBox::warning(
      topLevelWidget(),
      i18n("Find Again"),
      msg.data(),
      i18n("OK"),
      "",
      "",
      0,0 );
  }
  else
  {
    mFindDialog->findAgain();
  }

}



void CHexEditorWidget::replace( void )
{

  if( mReplaceDialog == 0 )
  {
    mReplaceDialog = new CReplaceDialog( 0, "replace" );
    if( mReplaceDialog == 0 ) { return; }
    mReplaceDialog->setCaption( i18n("Find and Replace") );
    connect( mReplaceDialog, SIGNAL(execute( USearchData & )), 
	     mHexView, SLOT(search( USearchData & )) );
  }
  if( mReplaceDialog->isVisible() == false )
  {
    centerDialog( mReplaceDialog, topLevelWidget() );
    mReplaceDialog->defaultFocus();
    mReplaceDialog->show();
  }
  else
  {
    mReplaceDialog->raise();
  }
}


void CHexEditorWidget::insertPattern( void )
{
  if( mInsertDialog == 0 )
  {
    mInsertDialog = new CInsertDialog( 0, "insert" );
    if( mInsertDialog == 0 ) { return; }
    mInsertDialog->setCaption( i18n("Insert pattern") );
    connect( mInsertDialog, SIGNAL(execute( SInsertData & )), 
	     mHexView, SLOT(insert( SInsertData & )) );
  }
  if( mInsertDialog->isVisible() == false )
  {
    centerDialog( mInsertDialog, topLevelWidget() );
    mInsertDialog->defaultFocus();
    mInsertDialog->show();
  }
  else
  {
    mInsertDialog->raise();
  }
}



void CHexEditorWidget::options( void )
{
  if( mOptionDialog == 0 )
  {
    const char *title = i18n( "General Preferences" );
    mOptionDialog = new COptionDialog( title, topLevelWidget() );
    if( mOptionDialog == 0 ) { return; }

    connect( mOptionDialog, SIGNAL(lineSizeChoise(const SDisplayLine &)),
	     SLOT(setLineSize(const SDisplayLine &)) );
    connect( mOptionDialog, SIGNAL(layoutChoise(const SDisplayLayout &)),
	     SLOT(setLayout(const SDisplayLayout &)) );
    connect( mOptionDialog, SIGNAL(fontChoise(const SDisplayFont &)),
	     SLOT(setFont(const SDisplayFont &)) );
    connect( mOptionDialog, SIGNAL(colorChoise(const SDisplayColor &)),
	     SLOT(setColor(const SDisplayColor &)) );
    connect( mOptionDialog, SIGNAL(cursorChoise(const SDisplayCursor &)),
	     SLOT(setCursor(const SDisplayCursor &)) );
    connect( mOptionDialog, SIGNAL(miscChoise(const SDisplayMisc &)),
	     SLOT(setMisc(const SDisplayMisc &)) );

    centerDialog( mOptionDialog, topLevelWidget() );
  }
  if( mOptionDialog->isVisible() == false )
  {
    centerDialog( mOptionDialog, topLevelWidget() );
    mOptionDialog->show();
  }
  else
  {
    mOptionDialog->raise();
  }
}



void CHexEditorWidget::readURL( QString &url, bool insert )
{
  //
  // 1) Make sure there is data in the url
  //
  if( url.isEmpty() == true )
  {
    return;
  }

  //
  // 2) Verify that the url is valid URL string. If not, try to repair it.
  // This will work it the url contains a name of a file in the
  // current directory.
  //
  KURL kurl( url.data() );
  if( kurl.isMalformed() == true )
  {
    QString tmp( "file:" );
    if( *url.data() != '/' )
    {
      tmp += QDir::currentDirPath() + "/";
    }
    tmp += url;
    kurl.parse( tmp );

    if( kurl.isMalformed() == true )
    {
      QString msg( i18n("Malformed URL") );     
      msg += "\n" + kurl.url();
      
      QMessageBox::warning(
	topLevelWidget(),
        i18n("Sorry"),
        msg,
        i18n("OK"),
        "",
        "",
        0, 0);
      return;
    }
  }

  //
  // 3) Load the file. Since this is an url we check if this is a local
  // or a "remote" file. The local file can be opened as a regular file.
  //
  if( kurl.isLocalFile() == true )
  {
    QString path( kurl.path() );
    KURL::decodeURL( path );
    readFile( path, kurl.url(), insert );
  }
  else
  {
    KFM *kfm = CreateKFMConnection();
    if( kfm == 0 )
    {
      return;
    }

    mKfmURL    = url.data();
    mKfmTMP.sprintf( "/tmp/""khexedit%u", time(0) );
    mKfmAction = CHexEditorWidget::GET;

    connect( kfm, SIGNAL(finished()),
             this, SLOT(kfmFinished()) );
    connect( kfm, SIGNAL(error(int, const char *)),
             this, SLOT(kfmError(int, const char *)) );

    emit internetLoadingChanged( true );
    kfm->copy( url.data(), mKfmTMP );

  }
}


void CHexEditorWidget::writeURL( QString &url ) 
{
  KURL kurl( url );

  if( kurl.isLocalFile() == true )
  {
    QString name( kurl.path() );
    KURL::decodeURL( name );
    if( writeFile( name ) == true ) 
    {
    }
  } 
  else 
  {
    mKfmURL    = url.data();
    mKfmTMP.sprintf( "/tmp/""khexedit%u", time(0) );
    mKfmAction = CHexEditorWidget::PUT;

    if( writeFile( mKfmTMP ) == false ) 
    {
      return;
    }

    KFM *kfm = CreateKFMConnection();
    if( kfm == 0 )
    {
      // FIXME unlink here
      return;
    }

    connect( kfm, SIGNAL(finished()),
             this, SLOT(kfmFinished()) );
    connect( kfm, SIGNAL(error(int, const char *)),
             this, SLOT(kfmError(int, const char *)) );

    kfm->copy( mKfmTMP, url.data() );
  }
}



bool CHexEditorWidget::readFile( const QString &diskPath, const QString &url,
				 bool insert ) 
{

  QFileInfo info( diskPath.data() );
  if( info.exists() == FALSE )
  {
    QMessageBox::warning(
      topLevelWidget(),
      i18n("Reading failed"),
      i18n("The specified file does not exist"),
      i18n("OK"),
      "",
      "",
      0,0);
    return( false );
  }

  if( info.isDir() == TRUE ) 
  {
    QMessageBox::warning(
      topLevelWidget(),
      i18n("Reading failed"),
      i18n("You have specified a directory."),
      i18n("OK"),
      "",
      "",
      0,0);
    return( false );
  }

  if( info.isReadable() == FALSE ) 
  {
    QMessageBox::warning(
      topLevelWidget(),
      i18n("Reading failed"),
      i18n("You do not have read permission to this file."),
      i18n("OK"),
      "",
      "",
      0,0);
    return( false );
  }

  QFile file( diskPath.data() );
  if( file.open( IO_ReadOnly | IO_Raw ) == FALSE ) 
  {
    QMessageBox::warning(
    topLevelWidget(),
    i18n("Reading failed"),
    i18n("An error occured while trying to open this file."),
    i18n("OK"),
    "",
    "",
    0,0);
    return( false );
  }

  mHexView->loadFile( &file, url, insert );
  file.close();

  return( true );
}



bool CHexEditorWidget::writeFile( QString &diskPath ) 
{
  QFileInfo info( diskPath.data() );
  if( info.exists() == true )
  { 
    if( info.isDir() == true ) 
    {
      QMessageBox::warning(
        topLevelWidget(),
	i18n("Writing failed"),
	i18n("You have specified a directory."),
	i18n("OK"),
	"",
	"",
	0,0 );
      return( false );
    }

    if( info.isWritable() == false ) 
    {
      QMessageBox::warning(
        topLevelWidget(),
	i18n("Writing failed"),
	i18n("You do not have write permission."),
	i18n("OK"),
	"",
	"",
	0,0);
      return( false );
    }
  }

  QFile file( diskPath.data() );
  if( file.open( IO_WriteOnly | IO_Raw ) == FALSE ) 
  {
    QMessageBox::warning(
    topLevelWidget(),
    i18n("Writing failed"),
    i18n("An error occured while trying to open this file."),
    i18n("OK"),
    "",
    "",
    0,0);
    return( false );
  }

  mHexView->saveFile( file );

  file.close();
  return( true );
}






KFM *CHexEditorWidget::CreateKFMConnection( void )
{
  if( mKfm != 0 )
  {
    QString msg;
    msg += i18n( "An internet job is already in progress." );
    msg += "\n\n";
    msg += i18n( "Please wait until it has completed" );
    msg += "\n";
    msg += i18n( "or stop it." );

    QMessageBox::information(
      topLevelWidget(),
      i18n("Operation failed"),
      i18n( msg ),
      i18n("OK"),
      "",
      "",
      0,0);
    return( 0 );
  }

  mKfm = new KFM;
  if( mKfm == 0 )
  {
    QString msg;
    msg += i18n( "Could not create KFM object." );
    msg += "\n";
    msg += i18n( "Insufficient memory." );

    QMessageBox::critical(
      topLevelWidget(),
      i18n("Operation failed"),
      msg.data(),
      i18n("OK"),
      "",
      "",
      0,0);
    return( 0 );
  }

  if( mKfm->isOK() == false ) 
  {
    QString msg;
    msg += i18n( "Could not create KFM object." );
    msg += "\n";
    msg += i18n( "Object is not valid." );

    QMessageBox::warning(
      topLevelWidget(),
      i18n("Operation failed"),
      msg.data(),
      i18n("OK"),
      "",
      "",
      0,0);
      delete mKfm; mKfm = 0;
      return( 0 );
  }

  
  return( mKfm ); 
}

void CHexEditorWidget::DestroyKFMConnection( void )
{
  delete mKfm; mKfm = 0;
}



void CHexEditorWidget::kfmFinished( void ) 
{
  emit internetLoadingChanged( false );
  if( mKfmAction == CHexEditorWidget::GET )
  {
    readFile( mKfmTMP, mKfmURL, false );
  }
  else if( mKfmAction == CHexEditorWidget::PUT )
  {
  }

  unlink( mKfmTMP.data() );
  DestroyKFMConnection();
}


void CHexEditorWidget::kfmError( int errCode, const char *errString ) 
{
  emit internetLoadingChanged( false );

  QString msg;
  msg += i18n( "KFM error" );
  msg += "\n";
  msg += errString;
  
  QMessageBox::critical(
    topLevelWidget(),
    i18n("Transfer failed"),
    msg.data(),
    i18n( "OK" ),
    "",
    "",
    0,0 );
}

CHexBuffer *CHexEditorWidget::documentItem( QString &url )
{ 
  for( CHexBuffer *hexBuffer = mDocumentList.first(); hexBuffer != 0; 
       hexBuffer = mDocumentList.next() )
  {
    if( hexBuffer->url() == url )
    {
      return( hexBuffer );
    }
  }
  
  return( 0 );
}


bool CHexEditorWidget::createBuffer( void )
{
  if( mHexView->valid() == false )
  {
    //
    // A buffer is already present, but it contains no data 
    //
    return( true );
  }

  CHexBuffer *hexBuffer = new CHexBuffer;
  if( hexBuffer == 0 ) 
  { 
    QString msg;
    msg += i18n( "Can not create text buffer." );
    msg += "\n";
    msg += i18n( "Insufficient memory." );

    QMessageBox::critical(
    topLevelWidget(),
    i18n("Loading failed" ),
    msg.data(),
    i18n( "OK" ),
    "",
    "",
    0,0 );
    return( false ); 
  }
  
  mDocumentList.append( hexBuffer );
  mHexView->setBuffer( hexBuffer );

  return( true );
}


void CHexEditorWidget::removeBuffer( void )
{
  mHexView->closeFile();

  if( mDocumentList.count() > 1 )
  {
    CHexBuffer *prev = 0;
    CHexBuffer *curr = mDocumentList.first();
    for( ; curr != 0; curr = mDocumentList.next() )
    {
      if( curr == mHexView->hexBuffer() )
      {
	CHexBuffer *ptr = prev != 0 ? prev : mDocumentList.next();
	if( ptr != 0 )
	{
	  mHexView->setBuffer( ptr );
	  mDocumentList.remove( curr );
	  break;
	}
      }
      prev = curr;
    }
  }

}


bool CHexEditorWidget::modified( void )
{
  for( CHexBuffer *hexBuffer = mDocumentList.first(); hexBuffer != 0; 
       hexBuffer = mDocumentList.next() )
  {
    if( hexBuffer->modified() == true )
    {
      return( true );
    }
  }
  
  return( false );
}




