#ifndef _OPTION_DIALOG_H_
#define _OPTION_DIALOG_H_

#include <qtabdlg.h>
#include <qframe.h>
#include <qlined.h>
#include <qcombo.h>
#include <qchkbox.h>
#include <kconfig.h>
#include <qradiobt.h>
#include <qpushbt.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qlayout.h> 

#include <kcolorbtn.h>
#include <ktabctl.h>

#include "kpropdlg.h"
#include "fontchooser.h"
#include "hexeditstate.h"



class CLineSizeWidget : public KTabCtl
{
  Q_OBJECT

  public:
    inline CLineSizeWidget( QWidget *parent = 0, const char *name = 0 );
    inline ~CLineSizeWidget( void );

    inline void setValue( const SDisplayLine &line );
    inline void getValue( SDisplayLine &line );

  protected:

  public slots:

  private:
  
  private:
    QFrame *page[4];
    QSpinBox *lineSpin[4];
    QSpinBox *columnSpin[4];
};


inline CLineSizeWidget::CLineSizeWidget( QWidget *parent, const char *name )
  :KTabCtl( parent, name )
{
  setBorder( false );

  const char *text[] = 
  {
    i18n("Hexadecimal"),
    i18n("Octal"),
    i18n("Binary"),
    i18n("Text only")
  };

  for( uint i=0; i<4;i++)
  {
    page[i] = new QFrame( this );
    if( page[i]  == 0 ) { return; }
    page[i]->setFrameStyle( QFrame::Box|QFrame::Sunken );
    addTab( page[i], text[i] ); 
    
    QGridLayout *gbox = new QGridLayout( page[i], 2, 2, 5 );
    lineSpin[i] = new QSpinBox( page[i] );
    lineSpin[i]->setFixedHeight( lineSpin[i]->sizeHint().height() );
    lineSpin[i]->setRange( 1, 1000 );
    gbox->addWidget( lineSpin[i], 0, 1 );
    columnSpin[i] = new QSpinBox( page[i] );
    columnSpin[i]->setFixedHeight( columnSpin[i]->sizeHint().height() );
    columnSpin[i]->setRange( 1, 1000 );
    gbox->addWidget( columnSpin[i], 1, 1 );
    QLabel *label1 = new QLabel(lineSpin[i],i18n("Default li&ne size [bytes]"),
				page[i] );
    gbox->addWidget( label1, 0, 0 );
    QLabel *label2 = new QLabel(columnSpin[i],i18n("Column &size"), 
				page[i] );
    gbox->addWidget( label2, 1, 0 );
    gbox->activate();
  }

  SDisplayLine line;
  setValue( line );

  show();
}

inline CLineSizeWidget::~CLineSizeWidget( void )
{
}

inline void CLineSizeWidget::setValue( const SDisplayLine &line )
{
  for( uint i=0; i<4; i++)
  {
    lineSpin[i]->setValue(line.lineSize[i]);
    columnSpin[i]->setValue(line.columnSize[i]);
  }
}

inline void CLineSizeWidget::getValue( SDisplayLine &line )
{
  for( uint i=0; i<4; i++)
  {
    line.lineSize[i] = lineSpin[i]->value();
    line.columnSize[i] = columnSpin[i]->value();
  }
}



class COptionDialog : public KPropDlg
{
  Q_OBJECT

  public:
    enum EPage
    {
      page_layout = 0,
      page_cursor,
      page_color,
      page_font,
      page_misc,
      page_max,
    };

    COptionDialog( const char *title, 
		   QWidget *parent = 0, char *name = 0, bool modal = FALSE );
    ~COptionDialog( void );

  public slots:

  signals:
    void lineSizeChoise( const SDisplayLine &lineSize );
    void layoutChoise( const SDisplayLayout &layout );
    void cursorChoise( const SDisplayCursor &cursor ); 
    void colorChoise( const SDisplayColor &color );
    void fontChoise( const SDisplayFont &font );
    void miscChoise( const SDisplayMisc &misc );

  protected slots:
    void slotBlinkIntervalCheck( bool state );
    void slotBlockCursorCheck( bool state );
    void slotFontSystem( bool state );
    void slotColorSystem( bool state );
    void slotDefault( void );
    void slotApply( void );

  protected:

  private:
    struct SLayoutWidgets
    {
      CLineSizeWidget *lineSize;
      QCheckBox *lockCheck;
      QCheckBox *splitCheck;
      QCheckBox *leftCheck;
      QCheckBox *rightCheck;
      QSpinBox  *separatorSpin;
      QSpinBox  *edgeSpin;
    };
    struct SCursorWidgets 
    {
      QCheckBox *blinkCheck;
      QSpinBox  *blinkSpin;
      QLabel    *blinkLabel;  
      QCheckBox *blockCheck;
      QCheckBox *thickCheck;
      QRadioButton *stopRadio;
      QRadioButton *hideRadio;
      QRadioButton *nothingRadio;
    };
    struct SColorWidgets
    {
      KColorButton *textBg;
      QCheckBox    *checkSystem;
      KColorButton *evenFg;
      KColorButton *oddFg;
      KColorButton *offsetFg;
      KColorButton *secondaryFg;
      KColorButton *separatorFg;
      KColorButton *inactiveBg;
      KColorButton *cursorFg;
      KColorButton *cursorBg;
      QLabel *evenLabelFg;
      QLabel *oddLabelFg;
      QLabel *offsetLabelFg;
      QLabel *secondaryLabelFg;
      QLabel *separatorLabelFg;
      QLabel *inactiveLabelFg;
      QLabel *cursorLabelFg;
      QLabel *cursorLabelBg;
    };
    struct SFontWidgets
    {
      QCheckBox    *checkSystem;
      KFontChooser *chooser;
    };
    struct SMiscWidgets
    {
      QSpinBox  *undoSpin;
      QCheckBox *autoCheck;
      QCheckBox *openCheck;
      QCheckBox *inputCheck;
      QCheckBox *fatalCheck;
    };

  private:
    void SetupLayoutPage( void );
    void SetupCursorPage( void );
    void SetupColorPage( void );
    void SetupFontPage( void);
    void SetupMiscPage( void );
 	  
  private:
    QFrame *mFrame[ page_max ];
    SLayoutWidgets mLayout;
    SCursorWidgets mCursor;
    SColorWidgets  mColor;
    SFontWidgets   mFont;
    SMiscWidgets   mMisc;


};





#endif







