#include <iostream.h>

#include <qmsgbox.h> 
#include <kapp.h>
#include <kiconloader.h>
#include <kstdaccel.h>

#include "toplevel.h"

static char *blurb = "Hex Editor 0.7 - Extended hex editor for KDE.\n\n"
                     "Author: Espen Sand\n" 
                     "espensa@online.no\n\n"
                     "This is a work in progress. If you\n\n"
                     "1) have any suggestions for improvement\n"
                     "2) have found a bug\n"
                     "3) just want to tell me how nice it is\n\n"
                     "then feel free to send me a mail.\n\n"
                     "Latest version is in KDE CVS and on\n"
                     "http://home.sol.no/~espensa/khexedit.";

QList<KHexEdit> KHexEdit::mWindowList;


KHexEdit::KHexEdit( void )
{
  mWindowList.append( this );
  mDocumentList.setAutoDelete( TRUE );
  mValid = false;

  //
  // Create main widget and register it.
  //
  mManager = new CHexManagerWidget( this, "manager" );
  if( mManager == 0 ) { return; }
  setView( mManager );


  connect( textView(), SIGNAL( cursorChanged( SCursorState & ) ),
	   this, SLOT( cursorChanged( SCursorState & ) ) );
  connect( textView(), SIGNAL( editMode( CHexBuffer::EEditMode ) ),
	   this, SLOT( editMode( CHexBuffer::EEditMode ) ) );
  connect( textView(), SIGNAL( textWidth( uint ) ),
	   this, SLOT( textWidth( uint ) ) );
  connect( textView(), SIGNAL( fileModified( SFileState & ) ),
	   this, SLOT( fileModified( SFileState & ) ) );
  connect( textView(), SIGNAL( layoutChanged( const SDisplayLayout & ) ),
	   this, SLOT( layoutChanged( const SDisplayLayout & ) ) );
  connect( textView(), SIGNAL( inputModeChanged( const SHexInputMode & ) ),
	   this, SLOT( inputModeChanged( const SHexInputMode & ) ) );
  connect( textView(), SIGNAL( bookmarkChanged( const QList<SCursorOffset> &)),
	   this, SLOT( bookmarkChanged( const QList<SCursorOffset> & ) ) );
  connect( textView(), SIGNAL( fileName( QString & ) ),
	   this, SLOT( fileActive( QString & ) ) );
  connect( textView(), SIGNAL( fileClosed( QString & ) ),
	   this, SLOT( fileClosed( QString & ) ) ); 
  connect( editor(), SIGNAL( internetLoadingChanged( bool ) ),
	   this, SLOT( internetLoadingChanged( bool ) ) );

  //
  // The entire window will act as a drop zone (wow, this is really 
  // cumbersome in Motif)
  //
  mZone = new KDNDDropZone( this, DndURL );
  if( mZone == 0 ) { return; }
  connect( mZone, SIGNAL( dropAction(KDNDDropZone *) ), 
	   this, SLOT( dropAction(KDNDDropZone *) ) );


  //
  // Make the editor popup menu.
  //
  mEditorPopup = new QPopupMenu;
  if( mEditorPopup == 0 ) { return; }
  connect( mEditorPopup, SIGNAL(activated (int)), SLOT( viewMenuCB (int) ));
  mBookmarkPopup = new QPopupMenu;
  if( mBookmarkPopup == 0 ) { return; }
  connect( mBookmarkPopup, SIGNAL(activated (int)),SLOT(bookmarkPopupCB(int)));


  mEditorPopup->setCheckable( TRUE );
  mEditorPopup->insertItem( i18n("&Open..."), editor(), SLOT(open()) );
  mEditorPopup->insertItem( i18n("&Save"), editor(), SLOT(save()) );
  mEditorPopup->insertItem( i18n("Save &as..."), editor(), SLOT(saveAs()) );
  mEditorPopup->insertSeparator();
  mEditorPopup->insertItem( i18n("&Bookmarks"), mBookmarkPopup, 0, 4 );
  mEditorPopup->insertSeparator();
  mPopHexMB = mEditorPopup->insertItem( i18n("&Hexadecimal"),view_hexadecimal);
  mPopOctalMB = mEditorPopup->insertItem( i18n("&Octal"), view_octal );
  mPopBinaryMB = mEditorPopup->insertItem( i18n("&Binary"), view_binary );
  mPopTextOnlyMB = mEditorPopup->insertItem( i18n("&Text only"), view_text );
  mEditorPopup->insertSeparator();
  mPopReadOnlyMB = mEditorPopup->insertItem( i18n("&Read only"),
					     permission_readonly );
  mPopResizeMB = mEditorPopup->insertItem( i18n("&Allow resize"),
					   permission_allowResize );
  textView()->setPopupMenu( mEditorPopup );


  //
  // Make the menu bar entries. We use the default entry for the help menu.
  //
  mRecent = new QPopupMenu;
  if( mRecent == 0 ) { return; }
  connect( mRecent, SIGNAL(activated(int)), SLOT(recentFile(int)) );

  mFilePermission = new QPopupMenu;
  if( mFilePermission == 0 ) { return; }

  mFile = new QPopupMenu;
  if( mFile == 0 ) { return; }

  mEditSpecial = new QPopupMenu;
  if( mEditSpecial == 0 ) { return; }

  mEdit = new QPopupMenu;
  if( mEdit == 0 ) { return; }

  mCharSet = new QPopupMenu;
  if( mCharSet == 0 ) { return; }

  mView = new QPopupMenu;
  if( mView == 0 ) { return; }

  mBookmark = new QPopupMenu;
  if( mBookmark == 0 ) { return; }

  mDocuments = new QPopupMenu;
  if( mDocuments == 0 ) { return; }
  connect( mDocuments, SIGNAL(activated(int)), SLOT(documentMenuCB(int)) );

  mTools = new QPopupMenu;
  if( mTools == 0 ) { return; }

  mOptions = new QPopupMenu;
  if( mOptions == 0 ) { return; }

  mHelp = kapp->getHelpMenu( true, blurb );

  //
  // Make the contents of the menus a connect the entries to a slot
  //
  KStdAccel *keys = new KStdAccel( kapp->getConfig() );
  if( keys == 0 ) { return; }

  mFilePermission->setCheckable( TRUE );
  mPermissionReadOnlyMB = mFilePermission->insertItem( i18n("&Read only"), 
		      permission_readonly );
  mPermissionResizeMB = mFilePermission->insertItem( i18n("&Allow resize"), 
		      permission_allowResize );
  connect( mFilePermission, SIGNAL(activated (int)), SLOT( viewMenuCB (int) ));

  mCharSet->setCheckable( TRUE );
  mCharSet->insertItem( i18n("&ISO 8859-1"), 0 );
  mCharSet->insertItem( i18n("&EBCDIC (CSCP 1047)"), 0 );
  mCharSet->insertItem( i18n("&Unicode"), 0 );
  

  mFile->insertItem( i18n("&New File"), 
		     editor(), SLOT(newFile()), keys->openNew() );
  mFile->insertItem( i18n("&Open..."), 
		     editor(), SLOT(open()), keys->open() );
  mFile->insertItem( i18n("&Insert..."), 
		     editor(), SLOT(insertFile()), CTRL+Key_I );
  mFile->insertItem( i18n("Open R&ecent"), mRecent );
  mFile->insertItem( i18n("&Reload"), 
		     editor(), SLOT(reload()) );
  mFile->insertSeparator();
  mFile->insertItem( i18n("&Save"), 
		     editor(), SLOT(save()), keys->save() );
  mFile->insertItem( i18n("Save &as..."), 
		     editor(), SLOT(saveAs()), CTRL+Key_A );
  mFile->insertItem( i18n("&Close File"), 
		     editor(), SLOT(close()), keys->close() );
  mFile->insertSeparator();
  mFile->insertItem( i18n("&Print..."), 
		     editor(), SLOT(print()), keys->print() );
  mFile->insertItem( i18n("E&xport..."), 
		     editor(), SLOT(export()) );
  mFile->insertSeparator();
  mFile->insertItem( i18n("Per&missions"), mFilePermission );
  mFile->insertItem( i18n("&Encoded as"), mCharSet );
  mFile->insertSeparator();
  mFile->insertItem( i18n("N&ew Window"), 
		     this, SLOT(newWindow()) );
  mFile->insertItem( i18n("Close &Window"), 
		     this, SLOT(closeWindow()) );
  mFile->insertSeparator();
  mFile->insertItem( i18n("&Quit"), 
		     this, SLOT(closeProgram()), keys->quit() );

  mEditUndoMB = mEdit->insertItem( i18n("&Undo"), 
		     editor(), SLOT(undo()), keys->undo() );
  mEditRedoMB = mEdit->insertItem( i18n("&Redo"), 
		     editor(), SLOT(redo()), CTRL+Key_Y );
  mEdit->insertSeparator();
  mEditCopyMB = mEdit->insertItem( i18n("&Copy"),
		     editor(), SLOT(copy()), keys->copy() );
  mEdit->insertItem( i18n("&Paste"),
		     editor(), SLOT(paste()), keys->paste() );
  mEditCutMB = mEdit->insertItem( i18n("Cu&t"),
		     editor(), SLOT(cut()), keys->cut() );
  mEdit->insertItem( i18n("Spec&ial"), mEditSpecial );
  mEdit->insertSeparator();
  mEdit->insertItem( i18n("&Select all"), 
		     editor(), SLOT(selectAll()) );
  mEdit->insertItem( i18n("&Unselect"), 
		     editor(), SLOT(unselect()) );
  mEdit->insertSeparator();
  mEdit->insertItem( i18n("&Goto..."),
		     editor(), SLOT(gotoOffset()), CTRL+Key_G );
  mEdit->insertItem( i18n("&Find..."),
		     editor(), SLOT(find()), keys->find() );
  mEdit->insertItem( i18n("Find &Again"),
		     editor(), SLOT(findAgain()), Key_F3 );
  mEdit->insertItem( i18n("Find and R&eplace..."),
		     editor(), SLOT(replace()), keys->replace() );
  mEdit->insertItem( i18n("Insert pattern..."),
		     editor(), SLOT(insertPattern()), CTRL+Key_Insert );

  mEditSpecialCopyText1MB = mEditSpecial->insertItem( 
			    i18n("Copy as text (&primary field)"),
			    editor(), SLOT(copyPrimaryFormatted()) );
  mEditSpecialCopyText2MB = mEditSpecial->insertItem( 
			    i18n("Copy as text (&all fields)"),
			    editor(), SLOT(copyAllFormatted()) );
  mEditSpecial->insertSeparator();
  mEditSpecial->insertItem( i18n("Paste into new &file"),
			    editor(), SLOT(pasteNewFile()) );
  mEditSpecial->insertItem( i18n("Paste into new &window"),
			    this, SLOT(pasteNewWindow()) );

  mView->setCheckable( TRUE );
  mViewHexMB      = mView->insertItem( i18n("&Hexadecimal"), 
		     view_hexadecimal );
  mViewOctalMB    = mView->insertItem( i18n("&Octal"), 
		     view_octal );
  mViewBinaryMB   = mView->insertItem( i18n("&Binary"), 
		     view_binary );
  mViewTextOnlyMB = mView->insertItem( i18n("&Text only"), 
		     view_text );
  mView->insertSeparator();
  mViewOffsetColumnMB = mView->insertItem( i18n("Show o&ffset column"), 
		     view_offset_area );
  mViewTextAreaMB = mView->insertItem( i18n("Show t&ext area"), 
		     view_text_area );
  mView->insertSeparator();
  mViewOffsetAsDecimalMB = mView->insertItem( i18n("Offset as &decimal"), 
		     view_offset_as_decimal );
  mView->insertSeparator();
  mViewBigCaseDataMB = mView->insertItem( i18n("&Upper case (data)"), 
					  view_case_data );
  mViewBigCaseOffsetMB = mView->insertItem( i18n("U&pper case (offset)"),
					    view_case_offset );
  mView->insertSeparator();
  mViewLockWidthMB = mView->insertItem( i18n("&Lock editor width"), 
		     view_lock_width );
  connect( mView, SIGNAL(activated (int)), SLOT( viewMenuCB (int) ));

  mTools->insertItem( i18n("&Extract strings..."),
		      0 );
  mTools->insertItem( i18n("&Statistics..."),
		      0 );
  mTools->insertItem( i18n("&Binary filters..."),
		      0 );


  mBookmark->insertItem( i18n("&Add bookmark"), 0, 0 );
  mBookmark->setAccel( CTRL+Key_B, 0 );
  mBookmark->insertItem( i18n("&Replace bookmark"), 1, 1 );
  mBookmark->setAccel( CTRL+Key_E, 1 );
  mBookmark->insertItem( i18n("&Delete all"), 2, 2 );
  mBookmark->setAccel( CTRL+Key_U, 2 );
  connect( mBookmark, SIGNAL(activated (int)), SLOT( bookmarkMenuCB (int) ));
  
  mOptions->setCheckable( TRUE );
  mOptions->insertItem( i18n("&General preferences..."),
		        option_edit );
  mOptions->insertSeparator();
  mOptionToolBarMB   = mOptions->insertItem( i18n("Show &Tool Bar"), 
		        option_toolbar );
  mOptionStatusBarMB = mOptions->insertItem( i18n("Show &Status Bar"), 
		        option_statusbar );
  mOptionUtilityMB   = mOptions->insertItem( i18n("Show &Utilities"), 
		        option_utility );
  mOptions->insertSeparator();
  mOptions->insertItem( i18n("&Save configuration"), this,
			SLOT(writeConfiguration()) );
  connect( mOptions, SIGNAL(activated (int)), SLOT( optionMenuCB (int) ));

  //
  // HACK 1999-05-15-ES (KDE-1.1)
  // Here comes a little hack. The 'Document' menu does not contain any 
  // entries by default. The entries will be added later and shall contain
  // an accelerator (Ctrl+<number=1-9>). The menu entry is created once a
  // document is installed, but without the hack, no accelerator is created.
  // Experimenting has shown that the accelaretor becomes visible if the 
  // menubar goes from "top" to "floating". If I remove all entries, and then
  // add one again it will get the accelerator. So that is what I do here.
  // I add one entry and remove it once the 'Document' menu is inserted.
  // (See addDocument() below) 
  //
  mDocuments->insertItem( "dummy", 0, 0 );
  mDocuments->setAccel( CTRL+Key_1, 0 );

  //  
  // Define menu entries in the menu bar.
  //
  menuBar()->insertItem( i18n("&File"), mFile );
  menuBar()->insertItem( i18n("&Edit"), mEdit );
  menuBar()->insertItem( i18n("&View"), mView );
  menuBar()->insertItem( i18n("&Tools"), mTools );
  menuBar()->insertItem( i18n("&Bookmarks"), mBookmark );
  menuBar()->insertItem( i18n("&Documents"), mDocuments );
  menuBar()->insertItem( i18n("&Options"), mOptions );
  menuBar()->insertSeparator();
  menuBar()->insertItem( i18n("&Help"), mHelp );

  //
  // The dummy has now served its purpose (see above)
  //
  mDocuments->removeItemAt( 0 );

  
  //
  // Make the toolbar
  //
  QString pixmapPath;
  KIconLoader &loader = *kapp->getIconLoader();
  QPixmap pixmap;

  pixmap = loader.loadIcon( "filenew.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(newFile()), true,
			    i18n("New file") );
  pixmap = loader.loadIcon( "fileopen.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(open()), true,
			    i18n("Open file") );
  pixmap = loader.loadIcon( "reload.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(reload()), true,
			    i18n("Reload file") );
  pixmap = loader.loadIcon( "filefloppy.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(save()), true,
			    i18n("Save file") );
  pixmap = loader.loadIcon( "fileprint.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(print()), true,
			    i18n("Print file") );
  pixmap = loader.loadIcon( "search.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(find()), true,
			    i18n("Find") );
  pixmap = loader.loadIcon( "editcopy.xpm" );
  toolBar(0)->insertButton( pixmap, toolbar_Copy, SIGNAL(clicked()),
			    editor(), SLOT(copy()), true,
			    i18n("Copy") );
  pixmap = loader.loadIcon( "editpaste.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(paste()), true,
			    i18n("Paste") );
  pixmap = loader.loadIcon( "editcut.xpm" );
  toolBar(0)->insertButton( pixmap, toolbar_Cut, SIGNAL(clicked()),
			    editor(), SLOT(cut()), true,
			    i18n("Cut") );
  pixmap = loader.loadIcon( "undo.xpm" );
  toolBar(0)->insertButton( pixmap, toolbar_Undo, SIGNAL(clicked()),
			    editor(), SLOT(undo()), true,
			    i18n("Undo") );
  pixmap = loader.loadIcon( "redo.xpm" );
  toolBar(0)->insertButton( pixmap, toolbar_Redo, SIGNAL(clicked()),
			    editor(), SLOT(redo()), true,
			    i18n("Redo") );
  pixmap = loader.loadIcon( "flag.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(benchmark()), true,
			    i18n("Benchmark") );
  pixmap = loader.loadIcon( "stop.xpm" );
  toolBar(0)->insertButton( pixmap, toolbar_InternetStop, SIGNAL(clicked()),
			    editor(), SLOT(stop()), false,
			    i18n("Stop internet connection") );

  //
  // Prepare the status bar. It will display (from right) the offset
  // in the file, Save state ( a pixmap), Insert state, general messages
  //

  statusBar()->setInsertOrder( KStatusBar::RightToLeft );
  statusBar()->insertItem( "RW", 0 );
  statusBar()->insertItem( "Bit: F", 1 );
  statusBar()->insertItem( "Offset: FFFFFFFFFF", 2 );
  statusBar()->insertItem( "Size: FFFFFFFFFF", 3 );
  statusBar()->insertItem( "OVR", 4 );
  statusBar()->insertItem( "M", 5 );
  statusBar()->insertItem( "", 6 );

  statusBar()->setAlignment( 0, AlignCenter );
  statusBar()->setAlignment( 4, AlignCenter );
  statusBar()->setAlignment( 5, AlignCenter );
  statusBar()->changeItem( "Bit:", 1 );
  statusBar()->changeItem( "Offset:", 2 );
  statusBar()->changeItem( "Size:", 3 );
  statusBar()->changeItem( "", 5 );

  connect( statusBar(), SIGNAL(pressed( int )), 
	   this, SLOT(statusBarPressed( int )) );


  readConfiguration();
  initialize();

  mValid = true;
}


KHexEdit::~KHexEdit()
{
  delete mManager;
}


void KHexEdit::initialize( void )
{
  mUndoState = UNDO_OK;   // Reset in function below
  setUndoState( 0 ); 

  mSelectionValid = true; // Reset in function below
  setSelectionState( false );

  textView()->setLayout( editor()->layout() );
  textView()->setInputMode( mInputMode );
 
  mOptions->setItemChecked( mOptionToolBarMB, mShowToolBar );
  if( mShowToolBar == true )
  {
    enableToolBar(KToolBar::Show);
  }
  else
  {
    enableToolBar(KToolBar::Hide);
  }

  mOptions->setItemChecked( mOptionStatusBarMB, mShowStatusBar );
  if( mShowStatusBar == true )
  {
    enableStatusBar(KStatusBar::Show);
  }
  else
  {
    enableStatusBar(KStatusBar::Hide);
  }

  mOptions->setItemChecked( mOptionUtilityMB, mShowUtility );
  mManager->setShowUtil( mShowUtility );

  setGeometry( 0, 0, editor()->defaultTextWidth(), 500 );
}



void KHexEdit::statusBarPressed( int id )
{
  if( id == 0 )
  {
    mInputMode.readOnly = mInputMode.readOnly == true ? false : true;
    textView()->setInputMode( mInputMode );
  }
  else if( id == 2 )
  {
    SDisplayLayout &layout = editor()->layout();
    if( layout.offsetMode == SDisplayLayout::hexadecimal )
    {
      layout.offsetMode = SDisplayLayout::decimal;
    }
    else
    {	
      layout.offsetMode = SDisplayLayout::hexadecimal;
    }
    textView()->setLayout( layout );
  }
}



void KHexEdit::open( QStrList &fileList )
{
  //
  // We load in reverse to allow first document in the list to
  // become the active one. 
  //
  for( char *s = fileList.last(); s != 0; s = fileList.prev() )
  {
    QString url( s );
    editor()->open( url );
  }
}

void KHexEdit::addRecentFile( QString &fileName )
{
  if( fileName.isEmpty() == true )
  {
    return;
  }
  
  if( fileName.contains( i18n( "Untitled" ), false ) )
  {
    return;
  }

  uint recentCount = mRecent->count();
  uint i;

  for( i=0; i < recentCount; i++ )
  {
    const char *text = mRecent->text(i);
    if( text == 0 )
    {
      continue;
    }
 
    if( fileName == text )
    { 
      mRecent->removeItemAt(i);
    }
  }
  
  if( mRecent->count() >= 5 ) 
  {
    mRecent->removeItemAt( 4 );
  }

  mRecent->insertItem( fileName, 0, 0 );
  for( i = 0; i < 5; i++ ) 
  {
    mRecent->setId( i, i );
  }

}



void KHexEdit::recentFile( int id )
{
  const char *text = mRecent->text(id);
  if( text != 0 )
  {
    QString url( text );
    editor()->open( url );
  }
}


void KHexEdit::newWindow( void )
{
  KHexEdit *app = new KHexEdit;
  if( app == 0 || app->isValid() == 0 )
  {
    QString msg;
    msg += i18n( "Can not create new window." );
    msg += "\n";
    msg += i18n( "Insufficient memory." );

    QMessageBox::critical(
    topLevelWidget(),
    i18n("New window" ),
    msg.data(),
    i18n( "OK" ),
    "",
    "",
    0,0);
    return;
  }
  app->show();
}



void KHexEdit::pasteNewWindow( void )
{
  KHexEdit *app = new KHexEdit;
  if( app == 0 || app->isValid() == 0 )
  {
    QString msg;
    msg += i18n( "Can not create new window." );
    msg += "\n";
    msg += i18n( "Insufficient memory." );

    QMessageBox::critical(
    topLevelWidget(),
    i18n("New window" ),
    msg.data(),
    i18n( "OK" ),
    "",
    "",
    0,0);
    return;
  }
  app->show();
  app->editor()->pasteNewFile();
}



void KHexEdit::closeWindow( void )
{
  close();
}


void KHexEdit::closeProgram( void )
{
  if( mWindowList.count() > 1 )
  {
    for( KHexEdit *w = mWindowList.first(); w != 0; w = mWindowList.next() )
    {
      if( w->editor()->modified() == true )
      {
	QString msg;
	msg += i18n( "There are windows with unsaved modified documents." );
	msg += "\n";
	msg += i18n( "If you quit now, these modifications will be lost." );
	msg += "\n";
	msg += i18n( "Quit anyways?" );
      
	int reply = QMessageBox::warning( 
	  topLevelWidget(),
	  i18n( "Warning" ), 
	  msg.data(),
	  i18n( "&Yes" ),
	  i18n( "&No" ),
	  "",
	  1, 1 );
	if( reply == 0 ) // Yes
	{
	  break;
	}
	else
	{
	  return;
	}
      }
    }
    queryExit(); // Just to save some data
    kapp->quit();
  }
  else
  {
    closeWindow();
  }
}
    

bool KHexEdit::queryClose( void )
{
  bool success = editor()->closeAll();
  if( success == true )
  {
    mWindowList.removeRef( this );
  }

  return( success );
}


//
// We will always return true here. 
//
bool KHexEdit::queryExit( void )
{
  writeConfiguration();
  return( true );
}



void KHexEdit::readConfiguration( void )
{
  KConfig &config = *kapp->getConfig();
  readConfiguration( config );
}


void KHexEdit::writeConfiguration( void )
{
  KConfig &config = *kapp->getConfig();
  writeConfiguration( config );
}


void KHexEdit::writeConfiguration( KConfig &config )
{
  config.setGroup("General Options" );
  config.writeEntry("Width", width() );
  config.writeEntry("Height", height() );
  config.writeEntry("ShowToolBar", mShowToolBar );
  config.writeEntry("ShowStatusBar", mShowStatusBar );
  config.writeEntry("ShowUtility", mShowUtility );

  for( uint i=0; i < mRecent->count(); i++ )
  {
    const char *text = mRecent->text(i);
    if( text != 0 )
    {
      char buf[20];
      sprintf( buf, "Recent%d", i+1 );
      config.writeEntry( buf, text );
    }
  }

  util()->writeConfiguration( config );
  editor()->writeConfiguration( config );
}


void KHexEdit::readConfiguration( KConfig &config )
{
  config.setGroup("General Options");
  mShowToolBar   = config.readBoolEntry( "ShowToolBar", true );
  mShowStatusBar = config.readBoolEntry( "ShowStatusBar", true );
  mShowUtility   = config.readBoolEntry( "ShowUtility", true );
  for( uint i = 0; i < 5; i++ ) 
  {
    char buf[20];
    sprintf( buf, "Recent%d", i + 1 );
    QString fileName = config.readEntry( buf );
    if( fileName.isEmpty() == false )
    { 
      mRecent->insertItem( fileName );
    }
  }

  util()->readConfiguration( config );
  editor()->readConfiguration( config );
}



void KHexEdit::internetLoadingChanged( bool state )
{
  toolBar()->setItemEnabled( toolbar_InternetStop, state );
}




void KHexEdit::cursorChanged( SCursorState &state )
{
  QString bit( i18n("Bit:") );
  QString offset( i18n("Offset:") );

  if( state.valid == false )
  {
    statusBar()->changeItem( bit.data(), 1 );
    statusBar()->changeItem( offset.data(), 2 );
  }
  else
  {
    SDisplayLayout &layout = editor()->layout();
    char buf[20];
    
    sprintf( buf, " %u", state.cell );
    bit += buf;
    statusBar()->changeItem( bit.data(), 1 );

    if( layout.offsetMode == SDisplayLayout::hexadecimal )
    {
      if( layout.offsetUpperCase == false )
      {
	sprintf( buf, " %04x:%04x", state.offset>>16, state.offset&0x0000FFFF);
      }
      else
      {
	sprintf( buf, " %04X:%04X", state.offset>>16, state.offset&0x0000FFFF);
      }
    }
    else
    {
      sprintf( buf, " %010u", state.offset );
    }

    offset += buf;
    statusBar()->changeItem( offset.data(), 2 );
  }
  
  setUndoState( state.undoState );
  setSelectionState( state.selectionValid );
}


void KHexEdit::fileModified( SFileState &state )
{
  if( state.valid == true )
  {
    QString size;
    size.sprintf( "%s %u", i18n("Size:"), state.size );
    statusBar()->changeItem( size.data(), 3 );
    statusBar()->changeItem( state.modified == true ? "!" : "", 5 );
  }
  else
  {
    statusBar()->changeItem( "", 3 );
    statusBar()->changeItem( "", 5 );
  }
}



void KHexEdit::layoutChanged( const SDisplayLayout &layout )
{
  if( layout.primaryMode == SDisplayLayout::hexadecimal ) 
  {
    mView->setItemChecked( mViewHexMB, true );
    mEditorPopup->setItemChecked( mPopHexMB, true );
  }
  else
  {
    mView->setItemChecked( mViewHexMB, false );
    mEditorPopup->setItemChecked( mPopHexMB, false );
  }

  if( layout.primaryMode == SDisplayLayout::octal )
  { 
    mView->setItemChecked( mViewOctalMB, true );
    mEditorPopup->setItemChecked( mPopOctalMB, true );
  }
  else
  {
    mView->setItemChecked( mViewOctalMB, false );
    mEditorPopup->setItemChecked( mPopOctalMB, false );
  }

  if( layout.primaryMode == SDisplayLayout::binary )
  { 
    mView->setItemChecked( mViewBinaryMB, true );
    mEditorPopup->setItemChecked( mPopBinaryMB, true );
  }
  else
  {
    mView->setItemChecked( mViewBinaryMB, false );
    mEditorPopup->setItemChecked( mPopBinaryMB, false );
  }
  
  if( layout.primaryMode == SDisplayLayout::textOnly )
  { 
    mView->setItemChecked( mViewTextOnlyMB, true );
    mView->setItemEnabled( mViewTextAreaMB, false );
    mEditorPopup->setItemChecked( mPopTextOnlyMB, true );
  }
  else
  {
    mView->setItemChecked( mViewTextOnlyMB, false );
    mView->setItemEnabled( mViewTextAreaMB, true );
    mEditorPopup->setItemChecked( mPopTextOnlyMB, false );
  }

  mView->setItemChecked( mViewOffsetColumnMB, layout.offsetVisible );
  //mView->setItemEnabled( mViewBigCaseOffsetMB, layout.offsetVisible );

  if( layout.offsetMode == SDisplayLayout::hexadecimal )
  {
    mView->setItemChecked( mViewOffsetAsDecimalMB, false );
  }
  else
  {
    mView->setItemChecked( mViewOffsetAsDecimalMB, true );
  }

  if( layout.secondaryMode == SDisplayLayout::hide )
  {
    mView->setItemChecked( mViewTextAreaMB, false );
  }
  else
  {
    mView->setItemChecked( mViewTextAreaMB, true );
  }

  mView->setItemChecked( mViewBigCaseOffsetMB, layout.offsetUpperCase );
  mView->setItemChecked( mViewBigCaseDataMB, layout.primaryUpperCase );
  mView->setItemChecked( mViewLockWidthMB, layout.lockLine );

}


void KHexEdit::inputModeChanged( const SHexInputMode &mode )
{
  mFilePermission->setItemChecked( mPermissionReadOnlyMB, mode.readOnly );
  mFilePermission->setItemChecked( mPermissionResizeMB, mode.allowResize );
  mEditorPopup->setItemChecked( mPopReadOnlyMB, mode.readOnly );
  mEditorPopup->setItemChecked( mPopResizeMB, mode.allowResize );
  statusBar()->changeItem( mode.readOnly == true ? i18n("R") : i18n("RW"), 0 );
}



void KHexEdit::bookmarkChanged( const QList<SCursorOffset> &list )
{
  mBookmark->clear();
  mBookmarkPopup->clear();

  mBookmark->insertItem( i18n("&Add bookmark"), 0, 0 );
  mBookmark->setAccel( CTRL+Key_B, 0 );
  mBookmark->insertItem( i18n("&Replace bookmark"), 1, 1 );
  mBookmark->setAccel( CTRL+Key_E, 1 );
  mBookmark->insertItem( i18n("&Delete all"), 2, 2 );
  mBookmark->setAccel( CTRL+Key_U, 2 );
  if( list.count() == 0 )
  {
    return;
  }
  mBookmark->insertSeparator(3);

  char buf[30];
  for( uint i=0; i < list.count(); i++ )
  {
    const SCursorOffset *p = list.at(i);
    if( p == 0 ) { continue; }

    sprintf( buf, "Offset: %04X:%04X", p->offset>>16, p->offset&0x0000FFFF );
    mBookmark->insertItem( buf, i+4 );
    mBookmark->setId( i+4, i+4 );
    
    int key = acceleratorNumKey( i );
    if( key > 0 )
    {
      mBookmark->setAccel( ALT+key, i+4 );
    }

    sprintf( buf, "[%u] Offset: %04X:%04X", i+1, p->offset>>16, 
	     p->offset&0x0000FFFF );
    mBookmarkPopup->insertItem( buf );
  }

}


void KHexEdit::fileActive( QString &url )
{
  if( url.isNull() == true )
  {
    setCaption( kapp->getCaption() );
  }
  else
  {
    QString str( url );
    str += " (";
    str += kapp->getCaption();
    str += ")";
    setCaption( str.data() );

    addDocument( url );
    addRecentFile( url );
  }
}

void KHexEdit::fileClosed( QString &url )
{
  removeDocument( url );
}



void KHexEdit::editMode( CHexBuffer::EEditMode editMode )
{
  statusBar()->changeItem( editMode == CHexBuffer::edit_overwrite ? 
			   i18n("OVR") : i18n("INS"), 4 );
}


void KHexEdit::textWidth( uint width )
{
  setGeometry( x(), y(), editor()->defaultTextWidth(), height() );
}



void KHexEdit::dropAction( KDNDDropZone *dropZone )
{
  QStrList &list = dropZone->getURLList();
  open( list );
}


void KHexEdit::viewMenuCB( int id )
{
  SDisplayLayout &layout = editor()->layout();
  SDisplayLine &line = editor()->line();

  switch( id )
  {
    case view_hexadecimal:
      layout.primaryMode   = SDisplayLayout::hexadecimal;
      layout.lineSize      = line.lineSize[ SDisplayLine::hexadecimal ];
      layout.columnSize    = line.columnSize[ SDisplayLine::hexadecimal ];
      layout.columnSpacing = -1;
      layout.lockColumn    = true;
      layout.secondaryMode = SDisplayLayout::textOnly;
      textView()->setLayout( layout );
    break;

    case view_octal:
      layout.primaryMode   = SDisplayLayout::octal;
      layout.lineSize      = line.lineSize[ SDisplayLine::octal ];
      layout.columnSize    = line.columnSize[ SDisplayLine::octal ];
      layout.columnSpacing = -1;
      layout.lockColumn    = true;
      layout.secondaryMode = SDisplayLayout::textOnly;
      textView()->setLayout( layout );
    break;

    case view_binary:
      layout.primaryMode   = SDisplayLayout::binary;
      layout.lineSize      = line.lineSize[ SDisplayLine::binary ];
      layout.columnSize    = line.columnSize[ SDisplayLine::binary ];
      layout.columnSpacing = -1;
      layout.lockColumn    = true;
      layout.secondaryMode = SDisplayLayout::textOnly;
      textView()->setLayout( layout );
    break;

    case view_text:
      layout.primaryMode   = SDisplayLayout::textOnly;
      layout.lineSize      = line.lineSize[ SDisplayLine::textOnly ];
      layout.columnSize    = line.columnSize[ SDisplayLine::textOnly ];
      layout.columnSpacing = 0;
      layout.lockColumn    = false;
      layout.secondaryMode = SDisplayLayout::hide;
      textView()->setLayout( layout );
    break;

    case view_offset_area:
      layout.offsetVisible = layout.offsetVisible == true ? false : true;
      textView()->setLayout( layout );
    break;

    case view_text_area:
      if( layout.secondaryMode == SDisplayLayout::hide )
      {
	layout.secondaryMode = SDisplayLayout::textOnly;
      }
      else
      {
	layout.secondaryMode = SDisplayLayout::hide;
      }
      textView()->setLayout( layout );
    break;

    case view_offset_as_decimal:
      if( layout.offsetMode == SDisplayLayout::hexadecimal )
      {
	layout.offsetMode = SDisplayLayout::decimal;
      }
      else
      {	
	layout.offsetMode = SDisplayLayout::hexadecimal;
      }
      textView()->setLayout( layout );
    break;

    case view_case_data:
      if( layout.primaryUpperCase == true )
      {	
	layout.primaryUpperCase = false;
      }
      else
      {
	layout.primaryUpperCase = true;
      }
      textView()->setLayout( layout );
    break;

    case view_case_offset:
      if( layout.offsetUpperCase == true )
      {
	layout.offsetUpperCase = false;
      }
      else
      {
	layout.offsetUpperCase = true;
      }
      textView()->setLayout( layout );
    break;

    case view_lock_width:
      layout.lockLine = layout.lockLine == True ? False : True;
      textView()->setLayout( layout );
    break;

    case permission_readonly:
      mInputMode.readOnly = mInputMode.readOnly == true ? false : true;
      textView()->setInputMode( mInputMode );
    break;

    case permission_allowResize:
      mInputMode.allowResize = mInputMode.allowResize == true ? false : true;
      textView()->setInputMode( mInputMode );
    break;


  }
}


void KHexEdit::bookmarkMenuCB( int id )
{
  if( id == 0 )
  {
    editor()->addBookmark();
  }
  else if( id == 1 )
  {
    editor()->replaceBookmark();
  }
  else if( id == 2 )
  {
    editor()->removeBookmark();
  }
  else if( id >= 4 )
  {
    editor()->gotoBookmark( id - 4 );
  }
}


void KHexEdit::bookmarkPopupCB( int id )
{
  editor()->gotoBookmark( id );
}


void KHexEdit::optionMenuCB( int id )
{
  switch( id )
  {
    case option_toolbar:
      mShowToolBar = mShowToolBar == true ? false : true;
      mOptions->setItemChecked( mOptionToolBarMB, mShowToolBar );
      if( mShowToolBar == true )
      {
	enableToolBar(KToolBar::Show);
      }
      else
      {
	enableToolBar(KToolBar::Hide);
      }
    break;

    case option_statusbar:
      mShowStatusBar = mShowStatusBar == true ? false : true;
      mOptions->setItemChecked( mOptionStatusBarMB, mShowStatusBar );
      if( mShowStatusBar == true )
      {
	enableStatusBar(KStatusBar::Show);
      }
      else
      {
	enableStatusBar(KStatusBar::Hide);
      }
    break;

    case option_utility:
      mShowUtility = mManager->showUtil() == true ? false : true;
      mOptions->setItemChecked( mOptionUtilityMB, mShowUtility );
      mManager->setShowUtil( mShowUtility );
    break;

    case option_edit:
      editor()->options();
    break;
  }
}





void KHexEdit::setUndoState( uint undoState ) 
{
  if( mUndoState != undoState )
  {
    mUndoState = undoState;
    mEdit->setItemEnabled( mEditUndoMB, mUndoState & UNDO_OK );
    mEdit->setItemEnabled( mEditRedoMB, mUndoState & REDO_OK );
    toolBar()->setItemEnabled( toolbar_Undo, mUndoState & UNDO_OK );
    toolBar()->setItemEnabled( toolbar_Redo, mUndoState & REDO_OK );
  }
}


void KHexEdit::setSelectionState( bool selectionValid )
{
  if( mSelectionValid != selectionValid )
  {
    mSelectionValid = selectionValid;
    mEdit->setItemEnabled( mEditCopyMB, mSelectionValid );
    mEdit->setItemEnabled( mEditCutMB, mSelectionValid );
    mEditSpecial->setItemEnabled( mEditSpecialCopyText1MB, mSelectionValid );
    mEditSpecial->setItemEnabled( mEditSpecialCopyText2MB, mSelectionValid );

    toolBar()->setItemEnabled( toolbar_Copy, mSelectionValid );
    toolBar()->setItemEnabled( toolbar_Cut, mSelectionValid );
  }
}



void KHexEdit::documentMenuCB( int id )
{
  QString *entry = mDocumentList.at(id);
  if( entry != 0 )
  {
    editor()->selectDocument( *entry );
  }
}



void KHexEdit::addDocument( QString &fileName )
{
  uint documentCount = mDocumentList.count();
  for( uint i=0; i < documentCount; i++ )
  {
    QString *entry = mDocumentList.at(i);
    if( entry == 0 ) { continue; }

    if( fileName == *entry )
    {
      return;
    }
  }

  QString *newEntry = new QString( fileName.data() );
  if( newEntry == 0 ) { return; }

  mDocumentList.append( newEntry );

  mDocuments->insertItem( fileName, documentCount, documentCount );
  int key = acceleratorNumKey( documentCount );
  if( key > 0 )
  {
    mDocuments->setAccel( CTRL+key, documentCount );
  }
}



void KHexEdit::removeDocument( QString &fileName )
{
  uint documentCount = mDocumentList.count();
  for( uint i=0; i < documentCount; i++ )
  {
    QString *entry = mDocumentList.at(i);
    if( entry == 0 ) { continue; }

    if( fileName == *entry )
    {
      mDocumentList.remove( i );
      mDocuments->removeItemAt( i );

      for( i = 0; i < documentCount; i++ ) 
      {
	mDocuments->setId( i, i );
	int key = acceleratorNumKey( i );
	if( key > 0 )
	{
	  mDocuments->setAccel( CTRL+key, i );
	}
      }
      return;
    }
  }
}


